//
//  ViewController.swift
//  MapSample
//
//  Created by MasaKentaro on 2016/07/31.
//  Copyright © 2016年 MasaKentaro. All rights reserved.
//

import UIKit
import MapKit
import CoreMotion

class ViewController: UIViewController,
                      MKMapViewDelegate,
                      CLLocationManagerDelegate{

    var mapType: MKMapType = .standard
    var buildingEnabled: Bool = true
    var pitchEnabled: Bool = true
    var compassEnabled: Bool = true
    var scaleEnabled: Bool = true
    var updatingLocation: Bool = true
    var userLocationEnabled: Bool = true
    
    var locationManager: CLLocationManager!
    var coordinates: [CLLocationCoordinate2D] = []
    
    let altimeter = CMAltimeter()
    let pedometer = CMPedometer()
    
    @IBOutlet weak var mapView: MKMapView!
    @IBOutlet weak var navigationItemTitle: UINavigationItem!
    @IBOutlet weak var label: UILabel!
    @IBOutlet weak var altimeterLabel: UILabel!
    @IBOutlet weak var stepLabel: UILabel!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        
        let userDefaults = UserDefaults.standard
        
        userDefaults.register(defaults: ["mapType": 0])
        userDefaults.register(defaults: ["buildingEnabled": true])
        userDefaults.register(defaults: ["pitchEnabled": true])
        userDefaults.register(defaults: ["compassEnabled": true])
        userDefaults.register(defaults: ["scaleEnabled": true])
        userDefaults.register(defaults: ["updatingLocation": false])
        userDefaults.register(defaults: ["userLocationEnabled": false])
        
        switch userDefaults.integer(forKey:"mapType") {
        case 0:
            mapType = .standard
        case 1:
            mapType = .hybrid
        case 2:
            mapType = .satellite
        default:
            mapType = .standard
        }
        buildingEnabled = userDefaults.bool(forKey: "buildingEnabled")
        pitchEnabled = userDefaults.bool(forKey: "pitchEnabled")
        compassEnabled = userDefaults.bool(forKey: "compassEnabled")
        scaleEnabled = userDefaults.bool(forKey: "scaleEnabled")
        updatingLocation = userDefaults.bool(forKey:"updatingLocation")
        userLocationEnabled = userDefaults.bool(forKey: "userLocationEnabled")
        
        let centerCoordinate = CLLocationCoordinate2D(latitude:35.658581,
                                                      longitude:139.745433)
        let camera = MKMapCamera(lookingAtCenter: centerCoordinate,
                                 fromDistance: 700.0,
                                 pitch: 45.0,
                                 heading: 0.0)
        mapView.setCamera(camera, animated: true)
 
        locationManager = CLLocationManager()
        locationManager.delegate = self
        locationManager.distanceFilter = 100.0
        locationManager.desiredAccuracy = kCLLocationAccuracyHundredMeters
        
        mapView.delegate = self
        
        self.configureMapView()
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        self.configureMapView()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func configureMapView() {
        
        mapView.mapType = mapType
        mapView.camera.altitude = 300
        if pitchEnabled {
            mapView.camera.pitch = 45
        } else {
            mapView.camera.pitch = 0
        }
        
        mapView.showsBuildings = buildingEnabled
        mapView.showsCompass = compassEnabled
        mapView.showsScale = scaleEnabled
        mapView.showsUserLocation = userLocationEnabled
        if updatingLocation {
            locationManager.startUpdatingLocation()
        } else {
            locationManager.stopUpdatingLocation()
        }
    }
    
    
    func locationManager(_ manager: CLLocationManager,
                         didChangeAuthorization status: CLAuthorizationStatus) {
        switch status{
        case .authorizedWhenInUse:
            print("AuthorizedWhenInUse")
        case .authorized:
            print("Authorized")
        case .denied:
            print("Denied")
        case .restricted:
            print("Restricted")
        case .notDetermined:
            print("NotDetermined")
            locationManager.requestAlwaysAuthorization()
        }
    }
    
    func locationManager(_ manager: CLLocationManager,
                         didUpdateLocations locations: [CLLocation]) {
        
        let locations: NSArray = locations as NSArray
        let lastLocation: CLLocation = locations.lastObject as! CLLocation
        
        mapView.camera.centerCoordinate = lastLocation.coordinate
        
        self.configureMapView()
    }
    
    func mapView(_ mapView: MKMapView, didUpdate userLocation: MKUserLocation) {
        let latitude = Double(userLocation.coordinate.latitude)
        let longitude = Double(userLocation.coordinate.longitude)
        let newCoordinate = CLLocationCoordinate2D(latitude: latitude,
                                                   longitude: longitude)
        coordinates.append(newCoordinate)
        
        let polyline = MKPolyline(coordinates: &coordinates,
                                  count: coordinates.count)
        mapView.removeOverlays(mapView.overlays)
        mapView.add(polyline)
        
        label.text = String(format: "%f , %f (%d)",
                            userLocation.coordinate.latitude,
                            userLocation.coordinate.longitude,
                            coordinates.count)
        
        let geocorder = CLGeocoder()
        let location = CLLocation(latitude: latitude, longitude: longitude)
        
        geocorder.reverseGeocodeLocation(
            location,
            completionHandler: { (placemarks, error) -> Void in
                guard let placemarks = placemarks else {
                    return
                }
                
                for pm in placemarks {
                    
                    self.navigationItemTitle.title =
                        String(format:"%@ %@ %@ %@",
                               pm.administrativeArea ?? "",
                               pm.subAdministrativeArea ?? "",
                               pm.locality ?? "",
                               pm.name ?? "")
                    
                    print("name: \(pm.name)")
                    print("country: \(pm.country)")
                    print("ISOcountryCode: \(pm.isoCountryCode)")
                    print("administrativeArea: \(pm.administrativeArea)")
                    print("subAdministrativeArea: \(pm.subAdministrativeArea)")
                    print("locality: \(pm.locality)")
                    print("postalCode: \(pm.postalCode)")
                }
        })
    }
    
    func mapView(_ mapView: MKMapView, rendererFor overlay: MKOverlay)
        -> MKOverlayRenderer {
        
        let polylineRendere: MKPolylineRenderer =
            MKPolylineRenderer(overlay: overlay)
        polylineRendere.lineWidth = 5
        polylineRendere.strokeColor = UIColor.red
        
        return polylineRendere
    }
 
    @IBAction func altimeterButtonTapped(sender: UIBarButtonItem) {
        if (CMAltimeter.isRelativeAltitudeAvailable()) {
            altimeter.startRelativeAltitudeUpdates(to: OperationQueue.main,
                                                   withHandler:
                {data, error in
                    if error == nil {
                        if let data = data {
                            let pressure:NSNumber = data.pressure
                            let altitude:NSNumber = data.relativeAltitude
                            self.altimeterLabel.text =
                                String(format: "気圧：%.1f hPa, 相対高度:%.2f m",
                                       pressure.floatValue * 10.0,
                                       altitude.floatValue)
                        } else {
                            self.altimeterLabel.text = "データがありません"
                        }
                    }
            })
        } else {
            self.altimeterLabel.text = "CMAltimeterが使えません"
        }
    }
    
    @IBAction func stepButtonTapped(sender: UIBarButtonItem) {
        
        if CMPedometer.isStepCountingAvailable() {
            pedometer.startUpdates(from: Date()){
                (data: CMPedometerData?, error) -> Void in
                if error == nil {
                    if let data = data {
                        let steps = data.numberOfSteps
                        print( "Steps: \(steps)")
                        self.stepLabel.text = String(format: "%d 歩",steps)
                    } else {
                        self.stepLabel.text = "データがありません"
                    }
                }
            }
        } else {
            self.stepLabel.text = "CMPedometerが使えません"
        }
    }

    @IBAction func saveButtonTapped(sender: UIBarButtonItem) {
        
        let options = MKMapSnapshotOptions()
        options.region = mapView.region
        options.scale = UIScreen.main.scale
        options.size = CGSize(width: mapView.frame.width,
                              height: mapView.frame.height)
        
        let snapshotter = MKMapSnapshotter(options: options)
        
        snapshotter.start(completionHandler: {(snapshot, error) in
            
            if let snapshot = snapshot {
                UIImageWriteToSavedPhotosAlbum(
                    snapshot.image,
                    self,
                    #selector(ViewController.image(
                        image:didFinishSavingWithError:contextInfo:)), nil)
            }
        })
    }
    
    func image(image: UIImage, didFinishSavingWithError error: NSError!,
               contextInfo: UnsafeMutableRawPointer) {
        if error != nil {
            print(error.code)
            print("画像データの保存できませんでした")
        } else {
            print("画像データを保存しました")
        }
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?){
        
        let navigation:UINavigationController =
            segue.destination as! UINavigationController
        let viewController:SettingsViewController =
            navigation.visibleViewController as! SettingsViewController
        
        viewController.mapType = mapType
        viewController.buildingEnabled = buildingEnabled
        viewController.pitchEnabled = pitchEnabled
        viewController.compassEnabled = compassEnabled
        viewController.scaleEnabled = scaleEnabled
        viewController.updatingLocation = updatingLocation
        viewController.userLocationEnabled = userLocationEnabled
    }
    
    
    @IBAction func unwindToTop(segue: UIStoryboardSegue) {
    }
}

